@@ -3,35 +3,44 @@
 #include "RvcgIO.h"
 #include <Rcpp.h>
 #include <RvcgKD.h>
+#include <Rconfig.h>
+#ifdef SUPPORT_OPENMP
+#include <omp.h>
+#endif
 using namespace tri;
 using namespace Rcpp;
 
-RcppExport SEXP Rkdtree(SEXP vb0_, SEXP vb1_, SEXP k_) {
+RcppExport SEXP Rkdtree(SEXP vb0_, SEXP vb1_, SEXP k_ ,SEXP nofP_= wrap(16),SEXP mDepth_= wrap(64),SEXP threads_=wrap(1)) {
   try {
     int k = as<int>(k_);
+    int threads = as<int>(threads_);
+    unsigned int nofP = as<unsigned int >(nofP_);
+    unsigned int mDepth = as<unsigned int >(mDepth_);
     typedef pair<float,int> mypair;
     PcMesh target, query;
     Rvcg::IOMesh<PcMesh>::RvcgReadR(target, vb0_);  
     Rvcg::IOMesh<PcMesh>::RvcgReadR(query, vb1_);
  
-    List out = Rvcg::KDtree< PcMesh, PcMesh >::KDtreeIO(target, query, k);
+    List out = Rvcg::KDtree< PcMesh, PcMesh >::KDtreeIO(target, query, k,nofP, mDepth,threads);
     return out;
   } catch (std::exception& e) {
     ::Rf_error( e.what());
-    return wrap(1);
   } catch (...) {
     ::Rf_error("unknown exception");
   }
   
 }
-RcppExport SEXP RclosestKD(SEXP vb_, SEXP it_, SEXP ioclost_, SEXP itclost_, SEXP k_, SEXP sign_, SEXP smooth_, SEXP barycentric_, SEXP borderchk_, SEXP nofP_= wrap(16),SEXP mDepth_= wrap(64),SEXP angdev_=wrap(0), SEXP wnorm_=wrap(true)) {
+
+RcppExport SEXP RclosestKD(SEXP vb_, SEXP it_, SEXP ioclost_, SEXP itclost_, SEXP k_, SEXP sign_, SEXP smooth_, SEXP barycentric_, SEXP borderchk_, SEXP nofP_= wrap(16),SEXP mDepth_= wrap(64),SEXP angdev_=wrap(0), SEXP wnorm_=wrap(true), SEXP facenormals_ = wrap(true), SEXP threads_=wrap(1)) {
   try {
     bool smooth = as<bool>(smooth_);
     bool barycentric = as<bool>(barycentric_);
     bool borderchk = as<bool>(borderchk_);
     bool wnorm = as<bool>(wnorm_);
+    bool facenormals = as<bool>(facenormals_);
     unsigned int nofP = as<unsigned int >(nofP_);
     unsigned int mDepth = as<unsigned int >(mDepth_);
+    int threads = as<int>(threads_);
     int k = as<int>(k_);
     bool sign = as<bool>(sign_);
     MyMesh target;
@@ -55,16 +64,21 @@ RcppExport SEXP RclosestKD(SEXP vb_, SEXP it_, SEXP ioclost_, SEXP itclost_, SEX
       tri::UpdateSelection<MyMesh>::FaceFromBorderFlag(target);
     }
     Rvcg::KDtree< MyMesh, PcMesh >::getBary(target, bary);
-    List indices = Rvcg::KDtree< PcMesh, MyMesh >::KDtreeIO(bary, query, k,nofP, mDepth);
+    List indices = Rvcg::KDtree< PcMesh, MyMesh >::KDtreeIO(bary, query, k,nofP, mDepth,threads);
     IntegerMatrix ktree = indices["index"];
     NumericMatrix iomat(3,query.vn), normals(3,query.vn);
     NumericMatrix barycoord(3,query.vn);
     IntegerVector border(query.vn), faceptr(query.vn);
     std::fill(border.begin(), border.end(),0);
-    MyMesh::VertexIterator vi = query.vert.begin();
+    //MyMesh::VertexIterator vi = query.vert.begin();
     NumericVector distout(query.vn);
     NumericVector angle(query.vn);
+#ifdef SUPPORT_OPENMP
+    omp_set_num_threads(threads);
+#endif
+#pragma omp parallel for schedule(static)
     for (int i = 0; i < query.vn; i++) {
+      MyMesh::VertexIterator vi = query.vert.begin()+i;
       Point3f clost;
       MyMesh::CoordType tt, tmpnorm;
       MyFace::ScalarType dist = 1e12;
@@ -114,11 +128,14 @@ RcppExport SEXP RclosestKD(SEXP vb_, SEXP it_, SEXP ioclost_, SEXP itclost_, SEX
 	}
       }
       distout[i] = distance_old;
-      // get normals at hit point
       tt = clost*0;
-      for (int j=0; j <3;j++) {
-      
-	if (&(target.face[faceptr[i]].V(j)->N())) {
+	    
+      // get normals at hit point
+      if (facenormals) {
+	tt = target.face[faceptr[i]].N();
+      } else {
+	for (int j=0; j <3;j++) {
+	  //if (&(target.face[faceptr[i]].V(j)->N())) {
 	  Point3f vdist = target.face[faceptr[i]].V(j)->P() - clost;
 	  float weight = sqrt(vdist.dot(vdist));
 	  if (weight > 0)
@@ -128,6 +145,7 @@ RcppExport SEXP RclosestKD(SEXP vb_, SEXP it_, SEXP ioclost_, SEXP itclost_, SEX
 	    
 	  tt +=(target.face[faceptr[i]].V(j)->N()*weight);
 	}
+	//}
       }
       float vl = sqrt(tt.dot(tt));
       if (vl > 0) {//check for zero length normals
@@ -160,7 +178,7 @@ RcppExport SEXP RclosestKD(SEXP vb_, SEXP it_, SEXP ioclost_, SEXP itclost_, SEX
 	if (target.face[faceptr[i]].IsS())
 	  border[i] = 1;
       }
-      ++vi;//update iterator
+     
     }
     return List::create(Named("iomat")=iomat,
 			Named("distance")= distout,
