@@ -8,7 +8,7 @@
 *                                                                    \      *
 * All rights reserved.                                                      *
 *                                                                           *
-* This program is free software; you can redistribute it and/or modify      *   
+* This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
@@ -64,14 +64,14 @@ namespace vcg {
 namespace tri {
 namespace io {
 
-/** 
+/**
 This class encapsulate a filter for opening stl (sterolitograpy) meshes.
 The stl format is quite simple and rather un-flexible. It just stores, in ascii or binary the, unindexed, geometry of the faces.
 */
 template <class SaveMeshType>
 class ExporterSTL
 {
-public:    
+public:
 typedef typename SaveMeshType::FaceType FaceType;
 typedef unsigned short CallBackSTLFaceAttribute(const SaveMeshType &m, const FaceType &f);
 
@@ -83,75 +83,75 @@ static int Save(SaveMeshType &m, const char * filename, const int &mask, CallBac
 static int Save(SaveMeshType &m, const char * filename , bool binary =true, int mask=0, const char *objectname=0, bool magicsMode=0)
 {
   typedef typename SaveMeshType::FaceIterator FaceIterator;
-	FILE *fp;
+    FILE *fp;
 
-	fp = fopen(filename,"wb");
-	if(fp==0)
-		return 1;
+    fp = fopen(filename,"wb");
+    if(fp==0)
+        return 1;
 
-	if(binary)
-	{
-		// Write Header
-		char header[128]="VCG                                                                                                  ";
-		if(objectname)	strncpy(header,objectname,80);
-		if(magicsMode)
-		{
-		  strncpy(header,"COLOR=XXX MATERIAL=AAA BBB CCC                                                                       ",80);
-		  for(int i=0;i<3;++i)
-		  {
-			header[0x06+i]=0x7f;
-			header[0x13+i]=0x7f;
-			header[0x17+i]=0x7f;
-			header[0x1b+i]=0x7f;
-		  }
-		}
-		fwrite(header,80,1,fp);
-		// write number of facets
-		fwrite(&m.fn,1,sizeof(int),fp); 
-		Point3f p;
-		unsigned short attributes=0;
-		for(FaceIterator fi=m.face.begin(); fi!=m.face.end(); ++fi) if( !(*fi).IsD() )
-		{
-			// For each triangle write the normal, the three coords and a short set to zero
-			p.Import(vcg::NormalizedNormal(*fi));
-			fwrite(p.V(),3,sizeof(float),fp);
- 
-			for(int k=0;k<3;++k){
-				p.Import((*fi).V(k)->P());
-				fwrite(p.V(),3,sizeof(float),fp);
-			}
-			if ((mask & Mask::IOM_FACECOLOR) && tri::HasPerFaceColor(m))
-			{
-			  if(magicsMode) attributes = 32768 | vcg::Color4b::ToUnsignedR5G5B5(fi->C());
-						else attributes = 32768 | vcg::Color4b::ToUnsignedB5G5R5(fi->C());
-			}
-			fwrite(&attributes,1,sizeof(short),fp);
-		}
-	}
-	else
-	{
-		if(objectname) fprintf(fp,"solid %s\n",objectname);
-		else fprintf(fp,"solid vcg\n");
+    if(binary)
+    {
+        // Write Header
+        char header[128]="VCG                                                                                                  ";
+        if(objectname)	strncpy(header,objectname,80);
+        if(magicsMode)
+        {
+          strncpy(header,"COLOR=XXX MATERIAL=AAA BBB CCC                                                                       ",80);
+          for(int i=0;i<3;++i)
+          {
+            header[0x06+i]=0x7f;
+            header[0x13+i]=0x7f;
+            header[0x17+i]=0x7f;
+            header[0x1b+i]=0x7f;
+          }
+        }
+        fwrite(header,80,1,fp);
+        // write number of facets
+        fwrite(&m.fn,1,sizeof(int),fp);
+        Point3f p;
+        unsigned short attributes=0;
+        for(FaceIterator fi=m.face.begin(); fi!=m.face.end(); ++fi) if( !(*fi).IsD() )
+        {
+            // For each triangle write the normal, the three coords and a short set to zero
+            p.Import(vcg::TriangleNormal(*fi).Normalize());
+            fwrite(p.V(),3,sizeof(float),fp);
 
-		Point3f p;
-		FaceIterator fi;	
-		for(fi=m.face.begin(); fi!=m.face.end(); ++fi) if( !(*fi).IsD() )
-		{
-	  	// For each triangle write the normal, the three coords and a short set to zero
-			p.Import(vcg::NormalizedNormal(*fi));
-			fprintf(fp,"  facet normal %13e %13e %13e\n",p[0],p[1],p[2]);
-			fprintf(fp,"    outer loop\n");
-			for(int k=0;k<3;++k){
-				p.Import((*fi).V(k)->P());
-				fprintf(fp,"      vertex  %13e %13e %13e\n",p[0],p[1],p[2]);			
-			}
-			fprintf(fp,"    endloop\n");
-			fprintf(fp,"  endfacet\n");
-		}
-		fprintf(fp,"endsolid vcg\n");
-	}
-	fclose(fp);
-	return 0;
+            for(int k=0;k<3;++k){
+                p.Import((*fi).V(k)->P());
+                fwrite(p.V(),3,sizeof(float),fp);
+            }
+            if ((mask & Mask::IOM_FACECOLOR) && tri::HasPerFaceColor(m))
+            {
+              if(magicsMode) attributes = 32768 | vcg::Color4b::ToUnsignedR5G5B5(fi->C());
+                        else attributes = 32768 | vcg::Color4b::ToUnsignedB5G5R5(fi->C());
+            }
+            fwrite(&attributes,1,sizeof(short),fp);
+        }
+    }
+    else
+    {
+        if(objectname) fprintf(fp,"solid %s\n",objectname);
+        else fprintf(fp,"solid vcg\n");
+
+        Point3f p;
+        FaceIterator fi;
+        for(fi=m.face.begin(); fi!=m.face.end(); ++fi) if( !(*fi).IsD() )
+        {
+        // For each triangle write the normal, the three coords and a short set to zero
+            p.Import(TriangleNormal(*fi).Normalize());
+            fprintf(fp,"  facet normal %13e %13e %13e\n",p[0],p[1],p[2]);
+            fprintf(fp,"    outer loop\n");
+            for(int k=0;k<3;++k){
+                p.Import((*fi).V(k)->P());
+                fprintf(fp,"      vertex  %13e %13e %13e\n",p[0],p[1],p[2]);
+            }
+            fprintf(fp,"    endloop\n");
+            fprintf(fp,"  endfacet\n");
+        }
+        fprintf(fp,"endsolid vcg\n");
+    }
+    fclose(fp);
+    return 0;
 }
 static const char *ErrorMsg(int error)
 {
@@ -160,7 +160,7 @@ static const char *ErrorMsg(int error)
   {
     stl_error_msg.resize(2 );
     stl_error_msg[0]="No errors";
-	  stl_error_msg[1]="Can't open file";
+      stl_error_msg[1]="Can't open file";
     }
 
   if(error>1 || error<0) return "Unknown error";
@@ -168,15 +168,15 @@ static const char *ErrorMsg(int error)
 };
 
 /*
-	returns mask of capability one define with what are the saveable information of the format.
+    returns mask of capability one define with what are the saveable information of the format.
 */
 static int GetExportMaskCapability()
 {
-	int capability = 0;			
-	capability |= vcg::tri::io::Mask::IOM_VERTCOORD;
-	capability |= vcg::tri::io::Mask::IOM_FACEINDEX;
-	capability |= vcg::tri::io::Mask::IOM_FACECOLOR;
-	return capability;
+    int capability = 0;
+    capability |= vcg::tri::io::Mask::IOM_VERTCOORD;
+    capability |= vcg::tri::io::Mask::IOM_FACEINDEX;
+    capability |= vcg::tri::io::Mask::IOM_FACECOLOR;
+    return capability;
 }
 
 
